package org.catais.kettle.plugin.ilivalidator;

import java.io.IOException;

import org.pentaho.di.core.Const;
import org.pentaho.di.core.exception.KettleException;
import org.pentaho.di.core.exception.KettleStepException;
import org.pentaho.di.core.row.RowMetaInterface;
import org.pentaho.di.core.row.ValueMetaInterface;
import org.pentaho.di.trans.Trans;
import org.pentaho.di.trans.TransMeta;
import org.pentaho.di.trans.step.*;
import org.apache.commons.io.FilenameUtils;
import org.interlis2.validator.Validator;
import ch.ehi.basics.settings.Settings;

public class ValidatorStep extends BaseStep implements StepInterface {
	
	private ValidatorStepData data;
	private ValidatorStepMeta meta;
	
	private Settings settings;
	private boolean logFile;

	public ValidatorStep(StepMeta s, StepDataInterface stepDataInterface, int c, TransMeta t, Trans dis) {
		super(s, stepDataInterface, c, t, dis);
	}

	public boolean processRow(StepMetaInterface smi, StepDataInterface sdi) throws KettleException {
		meta = (ValidatorStepMeta) smi;
		data = (ValidatorStepData) sdi;
				
		Object[] r = getRow(); // get row, blocks when needed!
		if (r == null) // no more input to be expected...
		{
			setOutputDone();
			return false;
		}

		if (first) {
			first = false;

			RowMetaInterface outputRowMeta = getInputRowMeta().clone();
        	meta.getFields(outputRowMeta, getStepname(), null, null, this); // additional fields is appended here
            data.setOutputRowMeta(outputRowMeta);
            
            
    		int LENGTH = outputRowMeta.size();
    		
    		for (int i=0; i < LENGTH; i++) {
    			ValueMetaInterface vm = outputRowMeta.getValueMeta(i);
				System.out.println(vm.getName());
				
    		}

            
            
    		settings = new Settings();
    		settings.setValue(Validator.SETTING_ILIDIRS, Validator.SETTING_DEFAULT_ILIDIRS);
    		
    		logFile = meta.getLogFile();
            
			logBasic("ilivalidator step initialized successfully");
		}

		try {			
			Object[] outputRow = runValidation(data.getOutputRowMeta(), r);
			putRow(data.getOutputRowMeta(), outputRow);
		} catch (KettleStepException ke) {
			log.logError("ValidatorStep", ke.getSuperMessage());
			setErrors(1);
			setOutputDone();
			return false;
		}
		return true;
	}

	private synchronized Object[] runValidation(RowMetaInterface rowMeta, Object[] row) throws KettleStepException {
		final int LENGTH = row.length;
		Object[] result = new Object[LENGTH];
		
		// TODO: field not found
		int idxTransferFileField = rowMeta.indexOfValue(meta.getTransferFileField());
		int idxResultField = rowMeta.indexOfValue(meta.getOutputField());
		int idxLogFileField = rowMeta.indexOfValue(meta.getLogFileOutputField());
		
		// this is where the validation happens
		String fileName = (String) row[idxTransferFileField];
		
		if (logFile) {
			String baseFileName = FilenameUtils.getFullPath(fileName) 
					+ FilenameUtils.getBaseName(fileName);
			String logFileName = baseFileName + ".log";
			
			settings.setValue(Validator.SETTING_LOGFILE, logFileName);
			
			row[idxLogFileField] = logFileName;
		}
	
		boolean ret = Validator.runValidation(fileName, settings);
		
		row[idxResultField] = ret;
		
		return row;
	}
	
	public boolean init(StepMetaInterface smi, StepDataInterface sdi) {
		meta = (ValidatorStepMeta) smi;
		data = (ValidatorStepData) sdi;

		return super.init(smi, sdi);
	}

	public void dispose(StepMetaInterface smi, StepDataInterface sdi) {
		meta = (ValidatorStepMeta) smi;
		data = (ValidatorStepData) sdi;

		super.dispose(smi, sdi);
	}

	// Run is were the action happens!
	public void run() {
		logBasic("Starting to run...");
		try {
			while (processRow(meta, data) && !isStopped()) {
				// Process rows
			}
		} catch (Exception e) {
			logError("Unexpected error : " + e.toString());
			logError(Const.getStackTracker(e));
			setErrors(1);
			stopAll();
		} finally {
			dispose(meta, data);
			logBasic("Finished, processing " + getLinesRead() + " rows");
			markStop();
		}
	}

}
