package org.catais.kettle.plugin.ilivalidator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.pentaho.di.core.Const;
import org.pentaho.di.core.exception.KettleException;
import org.pentaho.di.core.row.RowMeta;
import org.pentaho.di.core.row.RowMetaInterface;
import org.pentaho.di.trans.TransMeta;
import org.pentaho.di.ui.core.dialog.ErrorDialog;
import org.pentaho.di.ui.trans.step.BaseStepDialog;
import org.pentaho.di.trans.step.BaseStepMeta;
import org.pentaho.di.trans.step.StepDialogInterface;

import java.util.TreeSet;

import org.catais.kettle.plugin.ilivalidator.Messages;

public class ValidatorStepDialog extends BaseStepDialog implements StepDialogInterface {
	
	private ValidatorStepMeta input;
	
	private Label wlTransferFile;
	private FormData fdlTransferFile;
	private FormData fdTransferFile;
	private CCombo wTransferFileField;

	private Label wlLogFile;
	private FormData fdlLogFile;
	private FormData fdLogFile;
	private Button wLogFile;

	public ValidatorStepDialog(Shell parent, Object in, TransMeta transMeta, String sname) {
		super(parent, (BaseStepMeta) in, transMeta, sname);
		input = (ValidatorStepMeta) in;
	}
	
	@Override
	public String open() {
		Shell parent = getParent();
		Display display = parent.getDisplay();

		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MIN | SWT.MAX);
		props.setLook(shell);
		setShellImage(shell, input);

		ModifyListener lsMod = new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				input.setChanged();
			}
		};
		changed = input.hasChanged();

		FormLayout formLayout = new FormLayout();
		formLayout.marginWidth = Const.FORM_MARGIN;
		formLayout.marginHeight = Const.FORM_MARGIN;

		shell.setLayout(formLayout);
		shell.setText(Messages.getString("ilivalidator.shell.title")); 

		int middle = props.getMiddlePct();
		int margin = Const.MARGIN;
		
		// stepname line
		wlStepname = new Label(shell, SWT.RIGHT);
		wlStepname.setText(Messages.getString("ilivalidator.label.step_name")); 
		props.setLook(wlStepname);
		fdlStepname = new FormData();
		fdlStepname.left = new FormAttachment(0, 0);
		fdlStepname.right = new FormAttachment(middle, -margin);
		fdlStepname.top = new FormAttachment(0, margin);
		wlStepname.setLayoutData(fdlStepname);
		
		wStepname = new Text(shell, SWT.SINGLE | SWT.LEFT | SWT.BORDER);
		wStepname.setText(stepname);
		props.setLook(wStepname);
		wStepname.addModifyListener(lsMod);
		fdStepname = new FormData();
		fdStepname.left = new FormAttachment(middle, 0);
		fdStepname.top = new FormAttachment(0, margin);
		fdStepname.right = new FormAttachment(100, 0);
		wStepname.setLayoutData(fdStepname);

		// transfer file field
		wlTransferFile = new Label(shell, SWT.RIGHT);
		wlTransferFile.setText(Messages.getString("ilivalidator.label.transfer_file_field")); 
		props.setLook(wlTransferFile);
		fdlTransferFile = new FormData();
		fdlTransferFile.left = new FormAttachment(0, 0);
		fdlTransferFile.right = new FormAttachment(middle, -margin);
		fdlTransferFile.top = new FormAttachment(wStepname, margin);
		wlTransferFile.setLayoutData(fdlTransferFile);

		wTransferFileField = new CCombo(shell, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL | SWT.READ_ONLY);
		props.setLook(wTransferFileField);
		wTransferFileField.addModifyListener(lsMod);
		fdTransferFile = new FormData();
		fdTransferFile.left = new FormAttachment(middle, 0);
		fdTransferFile.right = new FormAttachment(100, 0);
		fdTransferFile.top = new FormAttachment(wStepname, margin);
		wTransferFileField.setLayoutData(fdTransferFile);
		fillTransferFileFieldsList(wTransferFileField);
		
		// logfile 
		wlLogFile = new Label(shell, SWT.RIGHT);
		wlLogFile.setText(Messages.getString("ilivalidator.label.logfile")); 
		props.setLook(wlLogFile);
		fdlLogFile = new FormData();
		fdlLogFile.left = new FormAttachment(0, 0);
		fdlLogFile.right = new FormAttachment(middle, -margin);
		fdlLogFile.top = new FormAttachment(wTransferFileField, margin);
		wlLogFile.setLayoutData(fdlLogFile);
		
		wLogFile = new Button(shell, SWT.CHECK);
		props.setLook(wLogFile);
		fdLogFile = new FormData();
		fdLogFile.left = new FormAttachment(middle, 0);
		fdLogFile.right = new FormAttachment(100, 0);
		fdLogFile.top = new FormAttachment(wTransferFileField, margin);
		wLogFile.setLayoutData(fdLogFile);

		// OK and cancel buttons
		wOK = new Button(shell, SWT.PUSH);
		wOK.setText(Messages.getString("System.Button.OK")); 
		wCancel = new Button(shell, SWT.PUSH);
		wCancel.setText(Messages.getString("System.Button.Cancel")); 

		BaseStepDialog.positionBottomButtons(shell, new Button[] { wOK, wCancel }, margin, wLogFile);

		// Add listeners
		lsCancel = new Listener() {
			public void handleEvent(Event e) {
				cancel();
			}
		};
		lsOK = new Listener() {
			public void handleEvent(Event e) {
				ok();
			}
		};

		wCancel.addListener(SWT.Selection, lsCancel);
		wOK.addListener(SWT.Selection, lsOK);

		lsDef = new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				ok();
			}
		};

		wStepname.addSelectionListener(lsDef);
		wTransferFileField.addSelectionListener(lsDef);

		// Detect X or ALT-F4 or something that kills this window...
		shell.addShellListener(new ShellAdapter() {
			public void shellClosed(ShellEvent e) {
				cancel();
			}
		});
	
		// Set the shell size, based upon previous time...
		setSize();

		getData();
		input.setChanged(changed);

		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}

		return stepname;
	}
	
	private void fillTransferFileFieldsList(CCombo combo) {
		RowMetaInterface inputfields = null;
		try {
			inputfields = transMeta.getPrevStepFields(stepname);
		} catch (KettleException e) {
			inputfields = new RowMeta();
			new ErrorDialog(shell,"Error", "Could not find the fields", e);
		}

		String[] fieldNames	= inputfields.getFieldNames();
		String[] fieldNamesAndTypes = inputfields.getFieldNamesAndTypes(0);
		TreeSet<String> stringField  = new TreeSet<String>();
		for (int i=0; i < fieldNames.length; i++) {
			if (fieldNamesAndTypes[i].toLowerCase().contains("string")) {
				stringField.add(fieldNames[i]);
			}
		}
		combo.setItems(stringField.toArray(new String[]{}));

		// set the default selection from loaded repo/xml
		String transferFileField = input.getTransferFileField();
		int existingSelection = combo.indexOf(transferFileField);
		if (existingSelection > -1) {
			combo.select(existingSelection);
		}
	}	

	// Read data and place it in the dialog
	public void getData() {
		wLogFile.setSelection(input.getLogFile());
		wStepname.selectAll();
	}

	private void cancel() {
		stepname = null;
		input.setChanged(changed);
		dispose();
	}
	
	// let the plugin know about the entered data
	private void ok() {
		stepname = wStepname.getText(); // return value
		input.setTransferFileField(wTransferFileField.getText());
		input.setLogFile(wLogFile.getSelection());
		dispose();
	}

}
