package org.catais.kettle.plugin.ilivalidator;

import java.util.List;
import java.util.Map;

import org.eclipse.swt.widgets.Shell;
import org.pentaho.di.core.*;
import org.pentaho.di.core.database.DatabaseMeta; 
import org.pentaho.di.core.exception.*;
import org.pentaho.di.core.exception.KettleValueException;
import org.pentaho.di.core.row.*;
import org.pentaho.di.core.row.value.ValueMetaBoolean;
import org.pentaho.di.core.row.value.ValueMetaString;
import org.pentaho.di.core.variables.VariableSpace;
import org.pentaho.di.core.xml.XMLHandler;
import org.pentaho.di.repository.*;
import org.pentaho.di.trans.*;
import org.pentaho.di.trans.step.*;
import org.pentaho.metastore.api.IMetaStore;
import org.pentaho.di.core.annotations.Step;
import org.w3c.dom.Node;

@Step(
	id = "ilivalidator",
	i18nPackageName = "org.catais.kettle.plugin.ilivalidator",
	name = "ilivalidator.name",
	description = "ilivalidator.description",
	categoryDescription = "i18n:org.pentaho.di.trans.step:BaseStep.Category.Validation",
	image = "icon.png",
	documentationUrl = "http://blog.sogeo.services"
)
public class ValidatorStepMeta extends BaseStepMeta implements StepMetaInterface {

	/** On which field is the transformation applied? **/
	private String transferFileField;
	
	/** Create log file **/
	private boolean logFile;
	
	/** Logfile output field **/
	private String logFileOutputField;
	
	/** Validation result field. **/
	private String outputField;


	public ValidatorStepMeta() {
		super(); 
		transferFileField = "";
	}
	
	public String getTransferFileField() {
		return transferFileField;
	}
	
	public void setTransferFileField(String transferFileField) {
		this.transferFileField = transferFileField;
	}
	
	public boolean getLogFile() {
		return logFile;
	}
	
	public void setLogFile(boolean logFile) {
		this.logFile = logFile;
	}
	
	public String getLogFileOutputField() {
		return logFileOutputField;
	}

	public void setLogFileOutputField(String logFileOutputField) {
		this.logFileOutputField = logFileOutputField;
	}

	public String getOutputField() {
		return outputField;
	}

	public void setOutputField(String outputField) {
		this.outputField = outputField;
	}

	@Override
	public String getXML() throws KettleValueException {
		String retval = "";
		retval += "		<transfer_file_field>" + getTransferFileField() + "</transfer_file_field>" + Const.CR;
		retval += "		<log_file>" + getLogFile() + "</log_file>" + Const.CR;
		retval += "		<log_file_output_field>" + getLogFileOutputField() + "</log_file_output_field>" + Const.CR;
		retval += "		<output_field>" + getOutputField() + "</output_field>" + Const.CR;
		return retval;
	}

	@Override
	public void loadXML(Node stepnode, List<DatabaseMeta> databases, Map<String, Counter> counters) throws KettleXMLException {
		try {
			setTransferFileField(Const.NVL(XMLHandler.getNodeValue(XMLHandler.getSubNode(stepnode, "transfer_file_field")),""));			
			setLogFile("true".equalsIgnoreCase( XMLHandler.getTagValue(stepnode, "log_file")));					
			setLogFileOutputField(Const.NVL(XMLHandler.getNodeValue(XMLHandler.getSubNode(stepnode, "log_file_output_field")),""));
			setOutputField(Const.NVL(XMLHandler.getNodeValue(XMLHandler.getSubNode(stepnode, "output_field")),""));			
		} catch (Exception e) {
			throw new KettleXMLException("Ilivalidator Plugin Unable to read step info from XML node", e);
		}
	}
	
	@Override
	public void getFields(RowMetaInterface r, String origin, RowMetaInterface[] info, StepMeta nextStep, VariableSpace space) {
		// append the outputField to the output
		ValueMetaInterface vResult = new ValueMetaBoolean(outputField);
		vResult.setOrigin(origin);
	    r.addValueMeta(vResult);
	    
	    ValueMetaInterface vLogFile = new ValueMetaString(logFileOutputField);
	    vLogFile.setOrigin(origin);
	    r.addValueMeta(vLogFile);
	}

	@Override
	public void check(List<CheckResultInterface> remarks, TransMeta transmeta, StepMeta stepMeta, RowMetaInterface prev, String input[], String output[], RowMetaInterface info) {
		CheckResult cr;

		// See if we have input streams leading to this step!
		if (input.length > 0) {
			cr = new CheckResult(CheckResult.TYPE_RESULT_OK, "Step is receiving info from other steps.", stepMeta);
			remarks.add(cr);
		} else {
			cr = new CheckResult(CheckResult.TYPE_RESULT_ERROR, "No input received from other steps!", stepMeta);
			remarks.add(cr);
		}	
	}
	
	@Override
	public Object clone() {
		Object retval = super.clone();
		return retval;
	}
	
	@Override
	public void setDefault() {
		logFileOutputField = "logfile";
		outputField = "valid";
	}
	
	@Override
	public void readRep(Repository rep, IMetaStore metaStore, ObjectId id_step, List<DatabaseMeta> database) throws KettleException {
		try
		{
			transferFileField = rep.getStepAttributeString(id_step, "transfer_file_field");
			logFile = rep.getStepAttributeBoolean(id_step, "log_file");
			logFileOutputField = rep.getStepAttributeString(id_step, "log_file_output_field"); 
			outputField = rep.getStepAttributeString(id_step, "output_field"); 
		}
		catch(Exception e)
		{
//			throw new KettleException(Messages.getString("IlivalidatorStep.Exception.UnexpectedErrorInReadingStepInfo"), e);
		}
	}

	@Override
	public void saveRep(Repository rep, IMetaStore metaStore, ObjectId id_transformation, ObjectId id_step) throws KettleException
	{
		try
		{
			rep.saveStepAttribute(id_transformation, id_step, "transfer_file_field", transferFileField);  
			rep.saveStepAttribute(id_transformation, id_step, "log_file", logFile);  
			rep.saveStepAttribute(id_transformation, id_step, "log_file_output_field", logFileOutputField); 
			rep.saveStepAttribute(id_transformation, id_step, "output_field", outputField); 
		}
		catch(Exception e)
		{
//			throw new KettleException(Messages.getString("Ilivalidator.Exception.UnableToSaveStepInfoToRepository") + id_step, e); 
		}
	}

	
	public StepDialogInterface getDialog(Shell shell, StepMetaInterface meta, TransMeta transMeta, String name) {
		return new ValidatorStepDialog(shell, meta, transMeta, name);
	}

	public StepInterface getStep(StepMeta stepMeta, StepDataInterface stepDataInterface, int cnr, TransMeta transMeta, Trans disp) {
		return new ValidatorStep(stepMeta, stepDataInterface, cnr, transMeta, disp);
	}

	public StepDataInterface getStepData() {
		return new ValidatorStepData();
	}
}
